@@ -192,7 +192,19 @@ void KateScrollBar::mousePressEvent(QMouseEvent *e)
     // delete text preview
     hideTextPreview();
 
+    if (e->button() == Qt::MidButton) {
+        m_middleMouseDown = true;
+    } else if (e->button() == Qt::LeftButton) {
+        m_leftMouseDown = true;
+    }
+
     if (m_showMiniMap) {
+        if (m_leftMouseDown) {
+            // if we show the minimap left-click jumps directly to the selected position
+            int newVal = (e->pos().y()-m_mapGroveRect.top()) / (double)m_mapGroveRect.height() * (double)(maximum()+pageStep()) - pageStep()/2;
+            newVal = qBound(0, newVal, maximum());
+            setSliderPosition(newVal);
+        }
         QMouseEvent eMod(QEvent::MouseButtonPress,
                          QPoint(6, minimapYToStdY(e->pos().y())),
                          e->button(), e->buttons(), e->modifiers());
@@ -201,12 +213,6 @@ void KateScrollBar::mousePressEvent(QMouseEvent *e)
         QScrollBar::mousePressEvent(e);
     }
 
-    if (e->button() == Qt::MidButton) {
-        m_middleMouseDown = true;
-    } else if (e->button() == Qt::LeftButton) {
-        m_leftMouseDown = true;
-    }
-
     m_toolTipPos = e->globalPos() - QPoint(e->pos().x(), 0);
     const int fromLine = m_viewInternal->toRealCursor(m_viewInternal->startPos()).line() + 1;
     const int lastLine = m_viewInternal->toRealCursor(m_viewInternal->endPos()).line() + 1;
@@ -657,7 +663,6 @@ void KateScrollBar::miniMapPaintEvent(QPaintEvent *e)
     QRect visibleRect = docRect;
     visibleRect.moveTop(visibleStart);
     visibleRect.setHeight(visibleEnd - visibleStart);
-    m_mapSliderRect = visibleRect;
 
     // calculate colors
     const QColor backgroundColor = m_view->defaultStyleAttribute(KTextEditor::dsNormal)->background().color();
